#!/usr/bin/env python3

""" Armenian script converter

This scripts allows the user to convert between the latinized transliteration
of Armenian and the Armenian Script itself with (almost) simple rules.
Types supported with the flag -t:
    - armenian or none: Armenian alphabet
    - maiscules: Armenian alphabet in maiscules only
    - iso:  ISO romanization
    - Classical:  Classical romanization

Usage
-----
pieoffice armenian <text>
pieoffice armenian <text> --type
    ----------------------------------------------------------------
    | a 	 ա | b	    բ | g	 գ | d	    դ | e	 ե |
    | ye	 ե | z      զ | ee	 է | e'     ը | t'	 թ | 
    | zh	 ժ | i	    ի | l	 լ | x	    խ | c	 ծ | 
    | k 	 կ | h      հ | j	 ձ | g.     ղ | l.	 ղ |
    | ch.	 ճ | m      մ | y	 յ | n      ն | sh	 շ |
    | o 	 ո | ch     չ | p	 պ | jh     ջ | r.	 ռ | 
    | s	         ս | v	    վ | t        տ | r	    ր | c'	 ց |
    | w          ւ | p'     փ | k'       ք | o'     օ | f 	 ֆ |
    | u	         ու| ev     և | ?	 ՞ | .      ։ | .'	 ՝ |
    | ;          ՟ | ;'     ՛ | !	 ՜ | ``     « | ''	 » |
    ----------------------------------------------------------------


"""

import re

ASCII_TO_ARMENIAN_SCRIPT_MINISCULES = [
        # Complex input (+1 char)
        (r"ew",     "և"),
        (r"ee",     "է"),
        (r"e'",     "ը"),
        (r"t'",     "թ"),
        (r"zh",     "ժ"),
        (r"[lg]'",  "ղ"),
        (r"ch'",    "չ"),
        (r"ch",     "ճ"),
        (r"sh",     "շ"),
        (r"jh",     "ջ"),
        (r"c'",     "ց"),
        (r"p'",     "փ"),
        (r"k'",     "ք"),
        (r"o'",     "օ"),
        (r"r'",     "ռ"),

        (r"a",      "ա"),
        (r"b",      "բ"),
        (r"g",      "գ"),
        (r"d",      "դ"),
        (r"e",      "ե"),
        (r"z",      "զ"),
        (r"i",      "ի"),
        (r"l",      "լ"),
        (r"x",      "խ"),
        (r"c",      "ծ"),
        (r"k",      "կ"),
        (r"h",      "հ"),
        (r"j",      "ձ"),
        (r"m",      "մ"),
        (r"y",      "յ"),
        (r"n",      "ն"),
        (r"o",      "ո"),
        (r"p",      "պ"),
        (r"s",      "ս"),
        (r"v",      "վ"),
        (r"t",      "տ"),
        (r"r",      "ր"),
        (r"w",      "ւ"),
        (r"f",      "ֆ"),
        (r"u",      "ու"),
]

ASCII_TO_ARMENIAN_SCRIPT_MAISCULES = [
        # Complex input (+1 char)
        (r"EE",     "Է"),
        (r"E'",     "Ը"),
        (r"T'",     "Թ"),
        (r"Z[Hh]",  "Ժ"),
        (r"[LG]'",  "Ղ"),
        (r"C[Hh]'", "Չ"),
        (r"C[Hh]",  "Ճ"),
        (r"S[Hh]",  "Շ"),
        (r"J[Hh]",  "Ջ"),
        (r"C'",     "Ց"),
        (r"P'",     "Փ"),
        (r"K'",     "Ք"),
        (r"O'",     "Օ"),
        (r"R'",     "Ռ"),

        (r"A",      "Ա"),
        (r"B",      "Բ"),
        (r"G",      "Գ"),
        (r"D",      "Դ"),
        (r"E",      "Ե"),
        (r"Z",      "Զ"),
        (r"I",      "Ի"),
        (r"L",      "Լ"),
        (r"X",      "Խ"),
        (r"C",      "Ծ"),
        (r"K",      "Կ"),
        (r"H",      "Հ"),
        (r"J",      "Ձ"),
        (r"M",      "Մ"),
        (r"Y",      "Յ"),
        (r"N",      "Ն"),
        (r"O",      "Ո"),
        (r"P",      "Պ"),
        (r"S",      "Ս"),
        (r"V",      "Վ"),
        (r"T",      "Տ"),
        (r"R",      "Ր"),
        (r"W",      "Ւ"),
        (r"F",      "Ֆ"),
        (r"U",      "ՈՒ")
]

ASCII_TO_ARMENIAN_SCRIPT_PUNCT = [
    (r"\?", "՞"),
    (r"\.", "։"),
    (r"\.'", "՝"),
    (r";", "՟"),
    (r";'", "՛"),
    (r"!", "՜"),
    (r"``", "«"),
    (r"''", "»")
]

ASCII_TO_ARMENIAN_ISO = [
         # Miniscules
        (r"ee",             r"ē"),
        (r"e'",             r"ë"),
        (r"o'",             r"ò"),
        (r"([ptk])'",       r"\1’"),
        (r"zh",             r"ž"),
        (r"[lg]'",          r"ġ"),
        (r"ch'",            r"č"),
        (r"ch",             r"č̣"),
        (r"sh",             r"š"),
        (r"jh",             r"ǰ"),
        (r"r'",             r"ṙ"),
        (r"c",              r"ç"),
        (r"ç'",             r"c’"),
        # Maiscules
        (r"EE",             r"Ē"),
        (r"E'",             r"Ë"),
        (r"O'",             r"Օ"),
        (r"([PTKC])'",      r"\1’"),
        (r"Z[Hh]",          r"Ž"),
        (r"[LG]'",          r"Ġ"),
        (r"C[Hh]'",         r"Č̣"),
        (r"C[Hh]",          r"Č"),
        (r"S[Hh]",          r"Š"),
        (r"J[Hh]",          r"ǰ"),
        (r"R'",             r"Ṙ"),
        (r"C",              r"Ç"),
]

ASCII_TO_ARMENIAN_CLASSICAL = [
         # Miniscules
        (r"ee",             r"ē"),
        (r"e'",             r"ə"),
        (r"o'",             r"ō"),
        (r"([ptkc])'",      r"\1ʿ"),
        (r"zh",             r"ž"),
        (r"[lg]'",          r"ł"),
        (r"ch'",            r"čʿ"),
        (r"ch",             r"č"),
        (r"sh",             r"š"),
        (r"jh",             r"ǰ"),
        (r"r'",             r"r̄"),
        # Maiscules
        (r"EE",             r"Ē"),
        (r"E'",             r"Ə"),
        (r"O'",             r"Ō"),
        (r"([PTKC])'",      r"\1ʿ"),
        (r"Z[Hh]",          r"Ž"),
        (r"[LG]'",          r"Ł"),
        (r"C[Hh]'",         r"Čʿ"),
        (r"C[Hh]",          r"Č"),
        (r"S[Hh]",          r"Š"),
        (r"J[Hh]",          r"ǰ"),
        (r"R'",             r"R̄")
]

ARMENIAN_SCRIPT_NUMBERS = [
        (r"9(\d\d\d)",   r"Ք\1"),
        (r"8(\d\d\d)",   r"Փ\1"),
        (r"7(\d\d\d)",   r"Ւ\1"),
        (r"6(\d\d\d)",   r"Ց\1"),
        (r"5(\d\d\d)",   r"Ր\1"),
        (r"4(\d\d\d)",   r"Տ\1"),
        (r"3(\d\d\d)",   r"Վ\1"),
        (r"2(\d\d\d)",   r"Ս\1"),
        (r"1(\d\d\d)",   r"Ռ\1"),
        (r"9(\d\d)",     r"Ջ\1"),
        (r"8(\d\d)",     r"Պ\1"),
        (r"7(\d\d)",     r"Չ\1"),
        (r"6(\d\d)",     r"Ո\1"),
        (r"5(\d\d)",     r"Շ\1"),
        (r"4(\d\d)",     r"Ն\1"),
        (r"3(\d\d)",     r"Յ\1"),
        (r"2(\d\d)",     r"Մ\1"),
        (r"1(\d\d)",     r"Ճ\1"),
        (r"9(\d)",       r"Ղ\1"),
        (r"8(\d)",       r"Ձ\1"),
        (r"7(\d)",       r"Հ\1"),
        (r"6(\d)",       r"Կ\1"),
        (r"5(\d)",       r"Ծ\1"),
        (r"4(\d)",       r"Խ\1"),
        (r"3(\d)",       r"Լ\1"),
        (r"2(\d)",       r"Ի\1"),
        (r"1(\d)",       r"Ժ\1"),
        (r"9",           r"Թ"),
        (r"8",           r"Ը"),
        (r"7",           r"Է"),
        (r"6",           r"Զ"),
        (r"5",           r"Ե"),
        (r"4",           r"Դ"),
        (r"3",           r"Գ"),
        (r"2",           r"Բ"),
        (r"1",           r"Ա"),
        (r"0",           r""),
]

class AsciiConverter:
    """
    Replace ASCII notation with an Unicode transliteration scheme for
    Classical Armenian.

    Attributes
    ----------
    scheme : str
        String with the name of the transliteration scheme to be used.

    script_set: list
        List with correspondences to be used in the conversion method.

    Methods
    -------
    __init__(self, scheme)
        Constructs the converter with the transliteration scheme.

        Parameters
        -----------
            scheme : str
                String with the name of the transliteration scheme to be used.
                Options: "armenian_alphabet", "armenian_maiscules",
                         "iso", "classical"
                Default: "armenian_alphabet"

    converter (self, ascii_string)
        Converts a ascii string to the converter's scheme.

        Parameters
        ----------
            ascii_string : str
                String with the text to be converted in ASCII notation.

    >>> from cltk.alphabet.arm.ascii_to_unicode import AsciiConverter
    >>> ascii_replace = AsciiConverter()
    >>> string = "Patasxani et hreshtakn ew asee c'na."
    >>> ascii_replace.converter(string)
    "Պատասխանի ետ հրեշտակն և ասէ ցնա։"
    >>> ascii_replace_iso = AsciiConverter("iso")
    >>> ascii_replace_iso.converter(string)
    "Patasxani et hreštakn ew asē c’na."
    >>> ascii_replace_classical = AsciiConverter("classical")
    >>> ascii_replace_classical.converter(string)
    "Patasxani et hreštakn ew asē cʿna."
    >>> ascii_replace_maiscules = AsciiConverter("armenian_maiscules")
    >>> ascii_replace_maiscules.converter(string)
    "ՊԱՏԱՍԽԱՆԻ ԵՏ ՀՐԵՇՏԱԿՆ ԵՒ ԱՍԷ ՑՆԱ։"
    """

    def __init__(self, scheme="armenian_alphabet"):
        if scheme == "armenian_alphabet":
            self.scheme = "armenian_alphabet"
            self.script_set = ASCII_TO_ARMENIAN_SCRIPT_MINISCULES\
                    + ASCII_TO_ARMENIAN_SCRIPT_MAISCULES + ASCII_TO_ARMENIAN_SCRIPT_PUNCT\
                    + ARMENIAN_SCRIPT_NUMBERS
        elif scheme == "armenian_maiscules":
            self.scheme = "armenian_maiscules"
            self.script_set = ASCII_TO_ARMENIAN_SCRIPT_MAISCULES + ASCII_TO_ARMENIAN_SCRIPT_PUNCT\
                    + ARMENIAN_SCRIPT_NUMBERS
        elif scheme == "iso":
            self.scheme = "iso"
            self.script_set = ASCII_TO_ARMENIAN_ISO
        elif scheme == "classical":
            self.scheme = "classical"
            self.script_set = ASCII_TO_ARMENIAN_CLASSICAL

    def converter(self, ascii_text):
        if self.scheme == "armenian_maiscules":
            output = ascii_text.upper()
        else:
            output = ascii_text

        for pair in self.script_set:
            output = re.sub(pair[0], pair[1], output)
        return output



def alpha_to_armenian(input_string):
    converter = AsciiConverter(scheme="armenian_alphabet")
    output = converter.converter(input_string)
    return(output)


def alpha_to_iso_armenian(input_string):
    converter = AsciiConverter(scheme="iso")
    output = converter.converter(input_string)
    return(output)

def alpha_to_classical_armenian(input_string):
    converter = AsciiConverter(scheme="classical")
    output = converter.converter(input_string)
    return(output)



""" Vedic - Devanagari and IAST

The scheme follows the Harvard-Kyoto Standard.
Accents should be placed as following for DEVANAGARI
    - anudāttaḥ:
        > hk_to_deva(anudAttaH - a=)
        + अनुदात्तः - अ॒
    - svaritaḥ:
        > hk_to_deva(svaritaH - a+)
        + स्वरितः - अ॑

Otherwise, use ´, /, `. \\ and =, ^
"""

import re

ASCII_HK_TO_DEVA = (
    (r"([\n ]|^)ai", r"\1ऐ"),
    (r"([\n ]|^)au", r"\1औ"),
    (r"([\n ]|^)a", r"\1अ"),
    (r"([\n ]|^)A", r"\1आ"),
    (r"([\n ]|^)i", r"\1इ"),
    (r"([\n ]|^)I", r"\1ई"),
    (r"([\n ]|^)u", r"\1उ"),
    (r"([\n ]|^)U", r"\1ऊ"),
    (r"([\n ]|^)e", r"\1ए"),
    (r"([\n ]|^)o", r"\1ओ"),
    (r"([\n ]|^)lR", r"\1लृ"),
    (r"([\n ]|^)lRR", r"\1लॄ"),
    (r"([\n ]|^)RR", r"\1ॠ"),
    (r"([\n ]|^)R", r"\1ऋ"),
    (r"ai", "V ै ै"),
    (r"au", "V ौ"),
    (r"a", "V "),
    (r"A", "V ा"),
    (r"i", "V ि"),
    (r"I", "V ी"),
    (r"u", "V ु"),
    (r"U", "V ू"),
    (r"e", "V े"),
    (r"o", "V ो"),
    (r"lRR", "V ॣ"),
    (r"lR", "V ॢ"),
    (r"RR", "V ॄ"),
    (r"R", "V ृ"),
    (r"khV ", "ख"),
    (r"ghV ", "घ"),
    (r"chV ", "छ"),
    (r"jhV ", "झ"),
    (r"ThV ", "ठ"),
    (r"DhV ", "ढ"),
    (r"thV ", "थ"),
    (r"dhV ", "ध"),
    (r"phV ", "फ"),
    (r"bhV ", "भ"),
    (r"kV ", "क"),
    (r"gV ", "ग"),
    (r"GV ", "ङ"),
    (r"cV ", "च"),
    (r"jV ", "ज"),
    (r"JV ", "ञ"),
    (r"TV ", "ट"),
    (r"DV ", "ड"),
    (r"NV ", "ण"),
    (r"tV ", "त"),
    (r"dV ", "द"),
    (r"nV ", "न"),
    (r"pV ", "प"),
    (r"bV ", "ब"),
    (r"mV ", "म"),
    (r"yV ", "य"),
    (r"rV ", "र"),
    (r"lV ", "ल"),
    (r"vV ", "व"),
    (r"zV ", "श"),
    (r"SV ", "ष"),
    (r"sV ", "स"),
    (r"LLV ", "ऴ"),
    (r"LV ", "ळ"),
    (r"hV ", "ह"),
    (r"kh", "ख्"),
    (r"gh", "घ्"),
    (r"ch", "छ्"),
    (r"jh", "झ्"),
    (r"Th", "ठ्"),
    (r"Dh", "ढ्"),
    (r"th", "थ्"),
    (r"dh", "ध्"),
    (r"ph", "फ्"),
    (r"bh", "भ्"),
    (r"k", "क्"),
    (r"g", "ग्"),
    (r"G", "ङ्"),
    (r"c", "च्"),
    (r"j", "ज्"),
    (r"J", "ञ्"),
    (r"T", "ट्"),
    (r"D", "ड्"),
    (r"N", "ण्"),
    (r"t", "त्"),
    (r"d", "द्"),
    (r"n", "न्"),
    (r"p", "प्"),
    (r"b", "ब्"),
    (r"m", "म्"),
    (r"y", "य्"),
    (r"r", "र्"),
    (r"l", "ल्"),
    (r"v", "व्"),
    (r"z", "श्"),
    (r"S", "ष्"),
    (r"s", "स्"),
    (r"LL", "ऴ्"),
    (r"L", "ळ्"),
    (r"h", "ह्"),

    (r"M", "\u0902"),
    (r"H", "\u0903"),
    (r"\\", "\u0951"),
    (r"=", "\u0952"),
    (r"&", "\u0901"),
    (r"'", "ऽ"),

    (r"V ", " "),

    (r"1", "१"),
    (r"2", "२"),
    (r"3", "३"),
    (r"4", "४"),
    (r"5", "५"),
    (r"6", "६"),
    (r"7", "७"),
    (r"8", "८"),
    (r"9", "९"),
    (r"0", "०"),

    (r"\|\|", "॥"),
    (r"\|", "।"),

    (r"(\u094d)[\u0951\u0952]\\", r"\1"),
    (r"\/[\u0951\u0952]", r""),
)


ASCII_HK_TO_IAST = (
    (r"A", "ā"),
    (r"I", "ī"),
    (r"U", "ū"),
    (r"lRR", "ḹ"),
    (r"lR", "ḷ"),
    (r"RR", "ṝ"),
    (r"R", "ṛ"),
    (r"T", "ṭ"),
    (r"D", "ḍ"),
    (r"G", "ṅ"),
    (r"J", "ñ"),
    (r"N", "ṇ"),
    (r"z", "ś"),
    (r"S", "ṣ"),
    (r"L", "l̠"),
    (r"\\", "\u0300"),
    (r"/", "\u0301"),
    (r"MM", "ṁ"),
    (r"M", "ṃ"),
    (r"H", "ḥ"),
    (r"&", "m̐")
)

ASCII_HK_TO_ISO = (
    (r"A", "ā"),
    (r"I", "ī"),
    (r"U", "ū"),
    (r"e", "ē"),
    (r"o", "ō"),
    (r"lRR", "l̥̄"),
    (r"lR", "l̥"),
    (r"RR", "r̥̄"),
    (r"R", "r̥"),
    (r"T", "ṭ"),
    (r"D", "ḍ"),
    (r"G", "ṅ"),
    (r"J", "ñ"),
    (r"N", "ṇ"),
    (r"z", "ś"),
    (r"S", "ṣ"),
    (r"L", "ḷ"),

    (r"MM", "ṃ"),
    (r"M", "ṁ"),
    (r"H", "ḥ"),
    (r"\\", "\u0300"),
    (r"/", "\u0301"),
    (r"&", "m̐")
)


def hkUdToHkAnu(hkUdStr):
    hkSyllab = hkToSyllables(hkUdStr)
    hkAcc = hkAccentuation(hkSyllab)

    hkSyllabArray = hkSyllab.split(".")
    hkAnuAcc = list(udToAnu(hkAcc))

    hkAnuStr = map(anudatta_apply, hkAnuAcc, hkSyllabArray)

    return("".join(list(hkAnuStr)))


def anudatta_apply(accent, syllabe):
    if accent == "A":
        return (syllabe + "=")
    elif accent == "S":
        return (syllabe + "\\")
    else:
        return syllabe.replace("/", "")


def hkToSyllables(hkStr):
    hkSyllab = hkStr
    hkSyllab = re.sub(r"([aeiouAEIOUR])", r"\1.", hkSyllab)
    hkSyllab = re.sub(r"a\.([ui])\.", r"a\1.", hkSyllab)
    hkSyllab = re.sub(r"\.\/", r"/.", hkSyllab)
    hkSyllab = re.sub(r"\.([HM&])", r"\1.", hkSyllab)
    hkSyllab = re.sub(r"R\.R", r"RR", hkSyllab)

    hkSyllab = re.sub(r"\.$", "", hkSyllab)

    return hkSyllab


def hkAccentuation(hkSyllab):
    hkAcc = []
    hkSyllab_list = hkSyllab.split(".")

    for syllabe in hkSyllab_list:
        if "/" in syllabe:
            hkAcc.append("U")
        else:
            hkAcc.append("B")
    return ("".join(hkAcc))


def udToAnu(udStr):
    """
     Converts a string of udatta marked syllables to
     an anudatta marked one. Notation:
     U = udatta
     A = anudatta
     S = svarita
     B = unmarked in udatta notation
     D = unmarked in anudatta notation

     Example:
     >>> udToAnu('BBUBBUUB')
     "AADSADDS"
    """

    anuStr = udStr

    anuStr = re.sub(r"BU",  "AU", anuStr)
    anuStr = re.sub(r"UB",  "US", anuStr)
    anuStr = re.sub(r"U",   "D", anuStr)

    while ("BA" in anuStr or "BD" in anuStr):
        anuStr = re.sub(r"^(B*)[BD](A)",    r"\1A\2", anuStr)
        anuStr = re.sub(r"\n(B*)[BD](A)",   r"\1A\2", anuStr)
        anuStr = re.sub(r"([ADSB])B([AD])", r"\1D\2", anuStr)
        anuStr = re.sub(r"B$", "D", anuStr)

    return anuStr


class AsciiConverter:
    def __init__(self, scheme="hk_to_deva", udatta_to_anudatta=True):
        self.udata_to_anudatta = udatta_to_anudatta
        if scheme == "hk_to_deva":
            self.scheme = "hk_to_deva"
            self.script_set = ASCII_HK_TO_DEVA
        elif scheme == "hk_to_iast":
            self.scheme = "hk_to_iast"
            self.script_set = ASCII_HK_TO_IAST
            self.udata_to_anudatta = False
        elif scheme == "hk_to_iso":
            self.scheme = "hk_to_iso"
            self.script_set = ASCII_HK_TO_ISO
            self.udata_to_anudatta = False

    def converter(self, ascii_text):
        if self.udata_to_anudatta and self.scheme == "hk_to_deva":
            output = hkUdToHkAnu(ascii_text)
        elif self.scheme == "hk_to_deva":
            output = "".join((filter(lambda x: x not in ["/"], ascii_text)))
        else:
            output = ascii_text

        for pair in self.script_set:
            output = re.sub(pair[0], pair[1], output)
        return output


if __name__ == "__main__":
    deva = "अ॒ग्निमी॑ळे पु॒रोहि॑तं य॒ज्ञस्य॑ दे॒वमृ॒त्विज॑म्"
    hk = "agni/mILe puro/hitaM yajJa/sya deva/mRtvi/jam"

    ascii_replace = AsciiConverter(
        scheme="hk_to_deva", udatta_to_anudatta=True)
    print(ascii_replace.converter(hk))
    print(ascii_replace.converter(hk) == deva)
    ascii_replace = AsciiConverter(
        scheme="hk_to_deva", udatta_to_anudatta=False
        )
    print(ascii_replace.converter(hk))





#! /usr/bin/env python3

""" Avestan script converter

If no optional argument is passed, the script converts to Avestan Script.
Else, you can pass the optional argument translit for converting to
latin script with macrons and diacritics for Avestan.
The typing scheme is as follows:

--------------------------------------------------------------------------
| a   a   𐬀  | A   ā   𐬁  ||  á   å  𐬂  | Á ā̊  𐬃  || ã  ą  𐬄 | ãã  ą̇   𐬅 |
| æ   ə   𐬆  | Æ   ə̄   𐬇  ||  e   e  𐬈  | E ē  𐬉  || o  o  𐬊 | O   ō   𐬋 |
| i   i   𐬌  | I   ī   𐬍  ||  u   u  𐬎  | U ū  𐬏  || k  k  𐬐 | x   x   𐬑 |
| X   x́   𐬒  | xw  x   𐬓  ||  g   g  𐬔  | G ġ  𐬕  || gh γ  𐬖 | c   č   𐬗 |
| j   ǰ   𐬘  | t   t   𐬙  ||  th  ϑ  𐬚  | d d  𐬛  || dh δ  𐬜 | T   t̰   𐬝 |
| p   p   𐬞  | f   f   𐬟  ||  b   b  𐬠  | B β  𐬡  || ng ŋ  𐬢 | ngh ŋ́   𐬣 |
| ngw ŋ   𐬤  | n   n   𐬥  ||  ñ   ń  𐬦  | N ṇ  𐬧  || m  m  𐬨 | M   m̨   𐬩 |
| Y   ẏ   𐬪  | y   y   𐬫  ||  v   v  𐬬  | r r  𐬭  || s  s  𐬯 | z   z   𐬰 |
| sh  š   𐬱  | zh  ž   𐬲  ||  shy š́  𐬳  | S ṣ̌  𐬴  || h  h  𐬵 |           |
--------------------------------------------------------------------------

"""
import re

ASCII_TO_AVESTAN_SCRIPT = [
    # Complex
    (r"a'",         "\U00010B02"),  # 𐬂  AVESTAN LETTER AO
    (r"A'",         "\U00010B03"),  # 𐬃  AVESTAN LETTER AAO
    (r"a~",         "\U00010B04"),  # 𐬄  AVESTAN LETTER AN
    (r"A~",         "\U00010B05"),  # 𐬅  AVESTAN LETTER AAN
    (r"a\.",        "\U00010B06"),  # 𐬆  AVESTAN LETTER AE
    (r"A\.",        "\U00010B07"),  # 𐬇  AVESTAN LETTER AEE
    (r"x'",         "\U00010B12"),  # 𐬒 AVESTAN LETTER XYE
    (r"xv",         "\U00010B13"),  # 𐬓 AVESTAN LETTER XVE
    (r"th",         "\U00010B1A"),  # 𐬚 AVESTAN LETTER THE
    (r"dh",         "\U00010B1C"),  # 𐬜 AVESTAN LETTER DHE
    (r"bh",         "\U00010B21"),  # 𐬡 AVESTAN LETTER BHE
    (r"ngv",        "\U00010B24"),  # 𐬤 AVESTAN LETTER NGVE
    (r"ng'",        "\U00010B23"),  # 𐬣 AVESTAN LETTER NGYE
    (r"ng",         "\U00010B22"),  # 𐬢 AVESTAN LETTER NGE
    (r"n'",         "\U00010B26"),  # 𐬦 AVESTAN LETTER NYE
    (r"gh",         "\U00010B16"),  # 𐬖 AVESTAN LETTER GHE
    (r"sh'",        "\U00010B33"),  # 𐬳 AVESTAN LETTER SHYE
    (r"sh",         "\U00010B31"),  # 𐬱 AVESTAN LETTER SHE
    (r"zh",         "\U00010B32"),  # 𐬲 AVESTAN LETTER ZHE
    (r"Sh",         "\U00010B34"),  # 𐬴 AVESTAN LETTER SSHE
    (r"g'",         "\U00010B15"),  # 𐬕 AVESTAN LETTER GGE

    (r"a",          "\U00010B00"),  # 𐬀  AVESTAN LETTER A
    (r"A",          "\U00010B01"),  # 𐬁  AVESTAN LETTER AA
    (r"e",          "\U00010B08"),  # 𐬈  AVESTAN LETTER E
    (r"E",          "\U00010B09"),  # 𐬉  AVESTAN LETTER EE
    (r"o",          "\U00010B0A"),  # 𐬊  AVESTAN LETTER O
    (r"O",          "\U00010B0B"),  # 𐬋  AVESTAN LETTER OO
    (r"i",          "\U00010B0C"),  # 𐬌  AVESTAN LETTER I
    (r"I",          "\U00010B0D"),  # 𐬍  AVESTAN LETTER II
    (r"u",          "\U00010B0E"),  # 𐬎  AVESTAN LETTER U
    (r"U",          "\U00010B0F"),  # 𐬏  AVESTAN LETTER UU
    (r"k",          "\U00010B10"),  # 𐬐 AVESTAN LETTER KE
    (r"x",          "\U00010B11"),  # 𐬑 AVESTAN LETTER XE
    (r"g",          "\U00010B14"),  # 𐬔 AVESTAN LETTER GE
    (r"G",          "\U00010B15"),  # 𐬕 AVESTAN LETTER GGE
    (r"c",          "\U00010B17"),  # 𐬗 AVESTAN LETTER CE
    (r"j",          "\U00010B18"),  # 𐬘 AVESTAN LETTER JE
    (r"t",          "\U00010B19"),  # 𐬙 AVESTAN LETTER TE
    (r"d",          "\U00010B1B"),  # 𐬛 AVESTAN LETTER DE
    (r"T",          "\U00010B1D"),  # 𐬝 AVESTAN LETTER TTE
    (r"p",          "\U00010B1E"),  # 𐬞 AVESTAN LETTER PE
    (r"f",          "\U00010B1F"),  # 𐬟 AVESTAN LETTER FE
    (r"b",          "\U00010B20"),  # 𐬠 AVESTAN LETTER BE
    (r"n",          "\U00010B25"),  # 𐬥 AVESTAN LETTER NE
    (r"N",          "\U00010B27"),  # 𐬧 AVESTAN LETTER NNE
    (r"m",          "\U00010B28"),  # 𐬨 AVESTAN LETTER ME
    (r"M",          "\U00010B29"),  # 𐬩 AVESTAN LETTER HME
    (r"Y",          "\U00010B2A"),  # 𐬪 AVESTAN LETTER YYE
    (r"y",          "\U00010B2B"),  # 𐬫 AVESTAN LETTER YE
    (r"v",          "\U00010B2C"),  # 𐬬 AVESTAN LETTER VE
    (r"r",          "\U00010B2D"),  # 𐬭 AVESTAN LETTER RE
    (r"l",          "\U00010B2E"),  # 𐬮 AVESTAN LETTER LE
    (r"s",          "\U00010B2F"),  # 𐬯 AVESTAN LETTER SE
    (r"z",          "\U00010B30"),  # 𐬰 AVESTAN LETTER ZE
    (r"h",          "\U00010B35"),  # 𐬵 AVESTAN LETTER HE
    # (r"<++>",       "\U00010B39"), # 𐬹 AVESTAN ABBREVIATION MARK
    # (r"<++>",       "\U00010B3A"), # 𐬺 TINY TWO DOTS OVER ONE DOT PUNCTUATION
    # (r"<++>",       "\U00010B3B"), # 𐬻 SMALL TWO DOTS OVER ONE DOT PUNCTUATION
    # (r"<++>",       "\U00010B3C"), # 𐬼 LARGE TWO DOTS OVER ONE DOT PUNCTUATION
    # (r"<++>",       "\U00010B3D"), # 𐬽 LARGE ONE DOT OVER TWO DOTS PUNCTUATION
    # (r"<++>",       "\U00010B3E"), # 𐬾 LARGE TWO RINGS OVER ONE RING PUNCTUATION
    # (r"<++>",       "\U00010B3F")  # 𐬿 LARGE ONE RING OVER TWO RINGS PUNCTUATION
]


ASCII_TO_AVESTAN_HOFFMAN = [
    # Complex
    (r"a'",         "å"),  # 𐬂  AVESTAN LETTER AOV
    (r"A'",         "ā̊"),  # 𐬃  AVESTAN LETTER AAO
    (r"a~",         "ą"),  # 𐬄  AVESTAN LETTER AN
    (r"A~",         "ą̇"),  # 𐬅  AVESTAN LETTER AAN
    (r"a\.",        "ə"),  # 𐬆  AVESTAN LETTER AE
    (r"A\.",        "ə̄"),  # 𐬇  AVESTAN LETTER AEE
    (r"x'",         "x́"),  # 𐬒 AVESTAN LETTER XYE
    (r"xv",         "xᵛ"),  # 𐬓 AVESTAN LETTER XVE
    (r"th",         "ϑ"),  # 𐬚 AVESTAN LETTER THE
    (r"dh",         "δ"),  # 𐬜 AVESTAN LETTER DHE
    (r"bh",         "β"),  # 𐬡 AVESTAN LETTER BHE
    (r"ngv",        "ŋᵛ"),  # 𐬤 AVESTAN LETTER NGVE
    (r"ng'",        "ŋ́"),  # 𐬣 AVESTAN LETTER NGYE
    (r"ng",         "ŋ"),  # 𐬢 AVESTAN LETTER NGE
    (r"n'",         "ń"),  # 𐬦 AVESTAN LETTER NYE
    (r"gh",         "γ"),  # 𐬖 AVESTAN LETTER GHE
    (r"sh'",        "š́"),  # 𐬳 AVESTAN LETTER SHYE
    (r"sh",         "š"),  # 𐬱 AVESTAN LETTER SHE
    (r"zh",         "ž"),  # 𐬲 AVESTAN LETTER ZHE
    (r"Sh",         "ṣ̌"),  # 𐬴 AVESTAN LETTER SSHE
    (r"g'",         "ġ"),  # 𐬕 AVESTAN LETTER GGE

    (r"A",          "ā"),  # 𐬁  AVESTAN LETTER AA
    (r"E",          "ē"),  # 𐬉  AVESTAN LETTER EE
    (r"O",          "ō"),  # 𐬋  AVESTAN LETTER OO
    (r"I",          "ī"),  # 𐬍  AVESTAN LETTER II
    (r"U",          "ū"),  # 𐬏  AVESTAN LETTER UU
    (r"G",          "ġ"),  # 𐬕 AVESTAN LETTER GGE
    (r"T",          "t̰"),  # 𐬝 AVESTAN LETTER TTE
    (r"N",          "ṇ"),  # 𐬧 AVESTAN LETTER NNE
    (r"M",          "m̨"),  # 𐬩 AVESTAN LETTER HME
    (r"Y",          "ẏ"),  # 𐬪 AVESTAN LETTER YYE
]


class AsciiConverter:
    """
    Replace ASCII notation with an Unicode transliteration scheme for
    Avestan.

    Attributes
    ----------
    scheme : str
        String with the name of the transliteration scheme to be used.

    script_set: list
        List with correspondences to be used in the conversion method.

    Methods
    -------
    __init__(self, scheme)
        Constructs the converter with the transliteration scheme.

        Parameters
        -----------
            scheme : str
                String with the name of the transliteration scheme to be used.
                Options: "script" and "roman-hoffman"

    converter (self, ascii_string)
        Converts a ascii string to the converter's scheme.

        Parameters
        ----------
            ascii_string : str
                String with the text to be converted in ASCII notation.

    >>> from cltk.alphabet.ave.ascii_to_unicode import AsciiConverter
    >>> ascii_replace = AsciiConverter()
    >>> string = "ahiiA. yAsA na.manghA. ustAnazastO."
    >>> ascii_replace.converter(string)
    """

    def __init__(self, scheme="script"):
        if scheme == "roman-hoffman":
            self.scheme = "roman-hoffman"
            self.script_set = ASCII_TO_AVESTAN_HOFFMAN
        else:
            self.scheme = "script"
            self.script_set = ASCII_TO_AVESTAN_SCRIPT

    def converter(self, ascii_text):
        output = ascii_text

        for pair in self.script_set:
            output = re.sub(pair[0], pair[1], output)
        return output


if __name__ == "__main__":
    ascii_replace = AsciiConverter()
    string = "ahiiA. yAsA na.manghA. ustAnazastO."
    print(ascii_replace.converter(string))
    ascii_replace = AsciiConverter("roman-hoffman")
    print(ascii_replace.converter(string))
